"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var sql = require('mssql');
var logger = log4js.getLogger('Sql');
var sqlConn;
function NLUMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.NLUMSSql = NLUMSSql;
function GetInteractionCampaigns(vcc) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .execute('GetInteractionCampaigns')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetInteractionCampaigns = GetInteractionCampaigns;
function GetInteractionAgents(vcc) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .execute('GetInteractionAgents')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetInteractionAgents = GetInteractionAgents;
function GetNLUs(vcc, search_criteria, search_expression, apps, isSystemApp) {
    logger.info('[NLUMSSql::GetNLUs] getting vcc:', vcc, 'search_criteria:', search_criteria, 'search_expression:', search_expression, 'apps:', apps);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .input('Apps', sql.NVarChar(sql.MAX), apps)
        .input('IsSystemApp', sql.Bit, isSystemApp)
        .execute('GetNLUs')
        .then(function (recordsets) {
        logger.info('[NLUMSSql::GetNLUs] getting recordsets:', recordsets.length);
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('[NLUMSSql::GetNLUs] getting recordset: ', recordset && recordset.length ? recordset.length : 0);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::GetNLUs] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.GetNLUs = GetNLUs;
function GetUniqueNLUByIntents(vcc, templateId, intents) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('TemplateId', sql.VarChar(100), templateId)
        .input('Intents', sql.NVarChar(sql.MAX), intents)
        .execute('GetUniqueNLUByIntents')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        if (recordset && recordset.length) {
            recordset = recordset.map(function (x) {
                return x.appId;
            });
        }
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::GetUniqueNLUByIntents] getting errorMsg:', errorMsg, 'vcc:', vcc, 'intents:', intents);
        return Promise.reject(err);
    });
}
exports.GetUniqueNLUByIntents = GetUniqueNLUByIntents;
function GetNLUsStudy(vcc, search_criteria, search_expression, names) {
    logger.info('[NLUMSSql::GetNLUsStudy] getting vcc:', vcc, 'search_criteria:', search_criteria, 'search_expression:', search_expression, 'names:', names);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .input('Names', sql.NVarChar(sql.MAX), names)
        .execute('GetNLUsStudy')
        .then(function (recordsets) {
        logger.info('[NLUMSSql::GetNLUsStudy] getting recordsets:', recordsets);
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('[NLUMSSql::GetNLUsStudy] getting recordset:', recordset);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::GetNLUsStudy] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.GetNLUsStudy = GetNLUsStudy;
function GetNLU(vcc, name) {
    logger.info('[NLUMSSql::GetNLU] getting vcc:', vcc, 'name:', name);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('GetNLU')
        .then(function (recordsets) {
        logger.info('[NLUMSSql::GetNLU] getting recordsets:', recordsets);
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('[NLUMSSql::GetNLU] getting recordset:', recordset);
        var nlu = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::GetNLU] getting nlu:', nlu);
        return Promise.resolve(nlu);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::GetNLU] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.GetNLU = GetNLU;
function GetNLUByName(vcc, name) {
    logger.info('[NLUMSSql::GetNLUByName] getting vcc:', vcc, 'name:', name);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('GetNLU')
        .then(function (recordsets) {
        logger.info('[NLUMSSql::GetNLUByName] getting recordsets:', recordsets);
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('[NLUMSSql::GetNLUByName] getting recordset:', recordset);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::GetNLUByName] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.GetNLUByName = GetNLUByName;
function GetNLUByAppId(vcc, appId) {
    logger.info('[NLUMSSql::GetNLUByAppId] getting vcc:', vcc, ', appId:', appId);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .execute('GetNLUByAppId')
        .then(function (recordsets) {
        logger.info('[NLUMSSql::GetNLUByAppId] getting recordsets: ', recordsets.length);
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var nlu = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(nlu);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::GetNLUByAppId] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.GetNLUByAppId = GetNLUByAppId;
function SaveNLU(vcc, name, culture, description, appId, subscriptionKey, isSystemApp, needTrain, isPublished, lastPublishedDate, isTrained, lastTrainedDate, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId) {
    logger.info('[NLUMSSql::SaveNLU] getting vcc:', vcc, 'name:', name, 'culture:', culture, 'description:', description, 'appId:', appId, 'subscriptionKey:', subscriptionKey, 'createdDate:', createdDate, 'createdByUserId:', createdByUserId, 'lastModifiedDate:', lastModifiedDate, 'lastModifiedByUserId:', lastModifiedByUserId);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .input('Culture', sql.VarChar(50), culture)
        .input('Description', sql.VarChar(50), description)
        .input('AppId', sql.VarChar(100), appId)
        .input('SubscriptionKey', sql.VarChar(50), subscriptionKey)
        .input('IsSystemApp', sql.Bit, isSystemApp)
        .input('NeedTrain', sql.Bit, needTrain)
        .input('IsPublished', sql.Bit, isPublished)
        .input('LastPublishedDate', sql.DateTime, lastPublishedDate)
        .input('IsTrained', sql.Bit, isTrained)
        .input('LastTrainedDate', sql.DateTime, lastTrainedDate)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .execute('SaveNLU')
        .then(function (recordsets) {
        logger.info('[NLUMSSql::SaveNLU] getting recordsets:', recordsets);
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('[NLUMSSql::SaveNLU] getting recordset:', recordset);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::SaveNLU] getting output:', output);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::SaveNLU] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveNLU = SaveNLU;
function DeleteNLU(vcc, name) {
    logger.info('[NLUMSSql::DeleteNLU] getting vcc:', vcc, 'name:', name);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('DeleteNLU')
        .then(function (recordsets) {
        logger.info('[NLUMSSql::DeleteNLU] getting recordsets:', recordsets);
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('[NLUMSSql::DeleteNLU] getting recordset:', recordset);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::DeleteNLU] getting output:', output);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::SaveNLU] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteNLU = DeleteNLU;
