"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var sql = require('mssql');
var logger = log4js.getLogger('Sql');
var sqlConn;
function UtteranceMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.UtteranceMSSql = UtteranceMSSql;
function GetUtterancesByIntentId(vcc, intentId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('IntentId', sql.VarChar(50), intentId)
        .execute('GetUtterancesByIntentId')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('GetUtterancesByIntentId: ', recordset);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetUtterancesByIntentId = GetUtterancesByIntentId;
function GetUtterancesByInteraction(vcc, audioId, section, part) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AudioId', sql.VarChar(50), audioId)
        .input('Section', sql.Numeric(5, 0), section)
        .input('Part', sql.Numeric(5, 0), part)
        .execute('GetUtterancesByInteraction')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('GetUtterancesByInteraction: ', recordset);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetUtterancesByInteraction = GetUtterancesByInteraction;
function SaveUtterance(vcc, appId, intentId, audioId, section, part, speaker, anchorOffset, focusOffset, currentTime, endCurrentTime, utteranceId, textContent, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('IntentId', sql.VarChar(50), intentId)
        .input('AudioId', sql.VarChar(50), audioId)
        .input('Section', sql.Numeric(5, 0), section)
        .input('Part', sql.Numeric(5, 0), part)
        .input('Speaker', sql.VarChar(3), speaker)
        .input('AnchorOffset', sql.Numeric(5, 0), anchorOffset)
        .input('FocusOffset', sql.Numeric(5, 0), focusOffset)
        .input('CurrentTime', sql.Float, currentTime)
        .input('EndCurrentTime', sql.Float, endCurrentTime)
        .input('UtteranceId', sql.Numeric(18, 0), utteranceId)
        .input('TextContent', sql.VarChar(500), textContent)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .execute('SaveUtterance')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('SaveUtterance - recordset: ', recordset);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('SaveUtterance - output: ', output);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveUtterance = SaveUtterance;
