"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var sql = require('mssql');
var logger = log4js.getLogger('Sql');
var sqlConn;
function VCCMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.VCCMSSql = VCCMSSql;
function GetVCCs(vcc, search_criteria, search_expression) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .execute('GetVCCs')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetVCCs = GetVCCs;
function GetIntegratedVCCs(vcc, ip, type) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('IP', sql.VarChar(50), ip)
        .input('Type', sql.Int, type)
        .execute('GetIntegratedVCCs')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetIntegratedVCCs = GetIntegratedVCCs;
function GetSubscriptionKeys(vcc) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .execute('GetSubscriptionKeys')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetSubscriptionKeys = GetSubscriptionKeys;
function GetSubscriptionKey(vcc) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .execute('GetSubscriptionKey')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var record = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(record);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetSubscriptionKey = GetSubscriptionKey;
function GetVCC(id) {
    return new sql.Request(sqlConn)
        .input('ID', sql.VarChar(50), id)
        .execute('GetVCC')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var record = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(record);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetVCC = GetVCC;
function SaveVCC(vcc, id, description, type, transcriptionDefaultLanguage, cognitiveEngine, country, language, timeZone, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(50), id)
        .input('Description', sql.VarChar(50), description)
        .input('Type', sql.VarChar(50), type)
        .input('TranscriptionDefaultLanguage', sql.VarChar(25), transcriptionDefaultLanguage)
        .input('CognitiveEngine', sql.VarChar(50), cognitiveEngine)
        .input('Country', sql.VarChar(50), country)
        .input('Language', sql.VarChar(50), language)
        .input('TimeZone', sql.VarChar(50), timeZone)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .execute('SaveVCC')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveVCC = SaveVCC;
function DeleteVCC(vcc, id) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(50), id)
        .execute('DeleteVCC')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteVCC = DeleteVCC;
function SaveVCCLicense(product, status, vcc, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Product', sql.VarChar(50), product)
        .input('Status', sql.Bit, status)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .execute('SaveVccLicense')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveVCCLicense = SaveVCCLicense;
