"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var moment = require("moment");
var request = require("request-promise-native");
var log4js = require("log4js");
var csvtojson = require('csvtojson');
var alasql = require('alasql');
var Client = require('ftp');
var SSH2Promise = require("ssh2-promise");
var mssql_service_1 = require("../db/mssql.service");
var file_manager_1 = require("../shared/file-manager");
var promises_shared_1 = require("../shared/promises.shared");
var config_1 = require("../config/config");
var config_2 = require("../config/config");
var integration_config_1 = require("../config/integration.config");
var utilities_1 = require("../shared/utilities");
var idatha_service_1 = require("./idatha.service");
var diarization_analysis_implementation_1 = require("../implementation/diarization-analysis.implementation");
var logger = log4js.getLogger('ServerScripts');
function AudioService(app) {
    app.get('/api/process/audio/get_audio/:processId/:id?', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var processId = req.params.processId;
        var id = req.params.id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (processId !== 'null') {
                var file_1 = config_2.PATH_FOLDER_PROCESS_AUDIO_PLAY + '\/' + processId + '\/' + id + '.mp3';
                return file_manager_1.BufferFile(file_1)
                    .then(function (result) {
                    return Promise.resolve(result);
                })
                    .catch(function (err) {
                    file_1 = config_2.PATH_FOLDER_PROCESS_AUDIO_PLAY + '\/' + processId + '\/' + id + '.wav';
                    return file_manager_1.BufferFile(file_1)
                        .then(function (result) {
                        return Promise.resolve(result);
                    });
                });
            }
            else {
                var url = id;
                url = Buffer.from(url, 'base64').toString();
                return ExternalRequest(url);
            }
        }).then(function (result) {
            res.json({ result: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/process/audio/get_audios_process', function (req, res, next) {
        logger.info('[AudioService::get_audios_process] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var newAudio = (req.body || {});
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var times = getTimes(newAudio.startTime, newAudio.endTime);
            return mssql_service_1.MSSql.GetAudiosProcess(newAudio.pageSize, newAudio.pageNumber, vcc, newAudio.processId, newAudio.id, newAudio.startDate, newAudio.endDate, times.startTime.hour, times.startTime.minute, times.endTime.hour, times.endTime.minute, newAudio.campaign, newAudio.agent, newAudio.resultCode, newAudio.type, newAudio.idathaResults, newAudio.totalIdathaResults, newAudio.interactionType, (newAudio.interactionIds || []).toString(), newAudio.folder);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[AudioService::get_audios_process] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/process/audio/get_audios_transcriptions', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var newAudio = (req.body || {});
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var times = getTimes(newAudio.startTime, newAudio.endTime);
            var interactionIds = (newAudio.interactionIds || []).toString();
            return mssql_service_1.MSSql.GetAudiosTranscriptions(newAudio.pageSize, newAudio.pageNumber, vcc, newAudio.processId, newAudio.id, newAudio.startDate, newAudio.endDate, times.startTime.hour, times.startTime.minute, times.endTime.hour, times.endTime.minute, newAudio.campaign, newAudio.agent, newAudio.resultCode, interactionIds);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/process/allegro/get_allegro_audios_statistic/:id', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var id = req.params.id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetAllegroAudiosStatistic(vcc, id);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/process/audio/get_audio_transcription/:process_id/:audio_id/:section/:part', function (req, res, next) {
        req.setTimeout(300000, function () { });
        logger.info('[AudioService::get_audio_transcription] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var processId = req.params.process_id === 'null' ? null : req.params.process_id;
        var interactionType = req.params.interactionType;
        var audioId = req.params.audio_id;
        var section = req.params.section * 1;
        var part = req.params.part * 1;
        var environment = null;
        var _resultReadFileContent = null;
        var _transcriptionQualification = 0;
        var buffer = null;
        var resultAudio;
        var interactionIdData;
        var fileNameAudio = '';
        var fileNameAnalysisSemantic = '';
        var fileNameAnalysisSemanticView = '';
        var jsonFileName = '';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (processId === null) {
                interactionIdData = utilities_1.getInteractionIdFromURLBase64(audioId);
                logger.info('[AudioService::get_audio_transcription] Getting interactionIdData:', interactionIdData, 'with req.params:', req.params, 'with vcc:', vcc, 'with resultAudio:', resultAudio);
                if (interactionIdData) {
                    audioId = interactionIdData.id ? interactionIdData.id : audioId;
                    section = section || section === 0 ? section : interactionIdData.section;
                    part = part || part === 0 ? part : interactionIdData.part;
                }
                else {
                    throw 'INVALID_AUDIO_DATA';
                }
            }
            return mssql_service_1.MSSql.GetAudio(vcc, processId, audioId, section, part);
        }).then(function (result) {
            resultAudio = result;
            if (resultAudio) {
                processId = processId ? processId : result.processId;
                return mssql_service_1.MSSql.GetProcess(vcc, processId);
            }
            else {
                return mssql_service_1.MSSql.GetEvaluationPlanByInteractionId(vcc, audioId);
            }
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_AUDIO_SYSTEM_SERVICE_NOT_FOUND';
            return mssql_service_1.MSSql.GetEnvironment(vcc, (result.environment || result.environmentId));
        }).then(function (result) {
            if (!result)
                throw 'ENVIRONMENT_NOT_FOUND';
            environment = result;
            return GetAudioFileFromAllegro(vcc, audioId, section, part, resultAudio && resultAudio.fileName ? resultAudio.fileName : null, null, null);
        }).catch(function (err) {
            return null;
        }).then(function (result) {
            if (result) {
                return Promise.resolve(result);
            }
            else {
                if (environment.type === 'allegro') {
                    return GetAudioFileFromAllegro(vcc, audioId, section, part, resultAudio && resultAudio.fileName ? resultAudio.fileName : null, null, null);
                }
                else if (environment.type === 'ftp') {
                    return GetAudioFileFromFtp(vcc, audioId, environment);
                }
                else if (environment.type === 'sftp') {
                    return GetAudioFileFromSftp(vcc, audioId, environment);
                }
            }
            return true;
        }).catch(function (err) {
            return null;
        }).then(function (result) {
            if (typeof result !== 'boolean') {
                if (result && result.audioFile && typeof result.audioFile === 'string') {
                    fileNameAudio = result && result.audioFile ? result.audioFile.split('/').pop() : null;
                }
                else {
                    fileNameAudio = null;
                }
                buffer = result && result.buffer ? result.buffer : null;
                if (resultAudio) {
                    if (resultAudio.fileName === null) {
                        var fileName = resultAudio.id;
                        if (resultAudio.section !== '' && resultAudio.part !== '') {
                            fileName += '_';
                            fileName += section;
                            fileName += '_';
                            fileName += part;
                        }
                        fileName += '.json';
                        resultAudio.fileName = fileName;
                    }
                    var regex = /\.(mp3|wav|json)$/;
                    jsonFileName = resultAudio.fileName.replace(regex, '.json');
                    fileNameAnalysisSemantic = config_1.SEMANCTIC_ANALYSIS_FOLDER;
                    fileNameAnalysisSemantic += '/';
                    fileNameAnalysisSemantic += vcc;
                    fileNameAnalysisSemantic += '/';
                    fileNameAnalysisSemantic += jsonFileName;
                    fileNameAnalysisSemanticView = config_1.SEMANCTIC_ANALYSIS_VIEW_FOLDER;
                    fileNameAnalysisSemanticView += '/';
                    fileNameAnalysisSemanticView += vcc;
                    fileNameAnalysisSemanticView += '/';
                    fileNameAnalysisSemanticView += jsonFileName;
                    logger.info('[AudioService::get_audio_transcription] Getting resultAudio:', resultAudio, 'before ExportAudioTranscription with req.params:', req.params, 'with vcc:', vcc, 'with resultAudio:', resultAudio);
                    logger.debug('[AudioService::get_audio_transcription] Searching raw transcription:', fileNameAnalysisSemantic);
                    logger.debug('[AudioService::get_audio_transcription] Searching view transcription:', fileNameAnalysisSemanticView);
                    if (resultAudio.init === null && resultAudio.isTranscribed === 1 && !resultAudio.isCancel) {
                        return ExportAudioTranscription(vcc, resultAudio, fileNameAnalysisSemantic, fileNameAnalysisSemanticView);
                    }
                }
            }
            return true;
        }).then(function (result) {
            logger.info('[AudioService::get_audio_transcription] Getting result:', result, 'with req.params:', req.params, 'with vcc:', vcc, 'with resultAudio:', resultAudio);
            if (fileNameAnalysisSemanticView && fileNameAnalysisSemanticView.length > 0) {
                return file_manager_1.ReadFileContent(fileNameAnalysisSemanticView, 'utf-8');
            }
            return '';
        }).then(function (result) {
            if (result && result.length > 0) {
                _resultReadFileContent = JSON.parse(result);
                var diarizationAnalysis = new diarization_analysis_implementation_1.DiarizationAnalysisImplementation();
                var phrases = diarizationAnalysis.GeneratePhrases(_resultReadFileContent.words, 1000000);
                _resultReadFileContent.chat = phrases.allPhrases;
            }
            if (resultAudio && resultAudio.processId) {
                var transcriptionDTO = {
                    vcc: vcc,
                    processId: resultAudio.processId,
                    id: resultAudio.id,
                    section: resultAudio.section,
                    part: resultAudio.part
                };
                return mssql_service_1.MSSql.GetTranscription(transcriptionDTO);
            }
            return true;
        }).then(function (result) {
            result = (result && result.StatusCodeError) ? null : result;
            if (typeof result !== 'boolean') {
                _transcriptionQualification = result ? result.value : null;
            }
            var data = {
                audioFile: fileNameAudio,
                channelsChanged: resultAudio && resultAudio.channelsChanged ? resultAudio.channelsChanged : false,
                buffer: buffer,
                resultTranscription: _resultReadFileContent,
                transcriptionQualification: _transcriptionQualification
            };
            if (res.headersSent)
                return;
            logger.debug('[AudioService::get_audio_transcription] Response data:', fileNameAnalysisSemanticView, data);
            res.json({ status: true, description: 'OK', data: data });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[AudioService::get_audio_transcription] Getting errorMsg:', errorMsg, 'with req.params:', req.params, 'with vcc:', vcc, 'with resultAudio:', resultAudio);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/process/audio/get_process_audio_summary/:processId', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var processId = req.params.processId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetProcessAudioSummary(vcc, processId);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/process/get_bulk_audio/:process_id', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var processId = req.params.process_id;
        var validAudios;
        var process;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetProcess(vcc, processId);
        }).then(function (resultProcess) {
            process = resultProcess;
            return GetValidAudios(vcc, process, userId);
        }).then(function (resultGetValidAudios) {
            validAudios = resultGetValidAudios;
            return mssql_service_1.MSSql.BulkAudioTemp(vcc, process, validAudios, userId);
        }).then(function (resultBulkAudioTemp) {
            var summary = alasql('SELECT Category1, Category2, Category3, Description, COUNT(*) AS Total FROM ? WHERE Description <> \'\' GROUP BY Category1, Category2, Category3, Description', [validAudios]);
            return mssql_service_1.MSSql.BulkAudioSummaryTemp(vcc, process, summary, userId);
        }).then(function (resultBulkAudioSummaryTemp) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: resultBulkAudioSummaryTemp });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/process/audio/get_audio_directory_count/:process_id', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var processId = req.params.process_id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return file_manager_1.GetFolderFiles(config_1.PATH_FOLDER_PROCESS_AUDIO + '/' + processId, ['csv'], ['wav', 'mp3'], true);
        }).then(function (resultGetFolderFiles) {
            if (res.headersSent)
                return;
            resultGetFolderFiles = JSON.parse(JSON.stringify(resultGetFolderFiles));
            resultGetFolderFiles = resultGetFolderFiles.filter(function (x) { return x.type !== 'directory'; });
            res.json({ status: true, description: 'OK', data: resultGetFolderFiles.length });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/process/audio/save_transcription', function (req, res, next) {
        logger.info('[AudioService::save_transcription] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var newTranscriptionDTO = (req.body || {});
        newTranscriptionDTO.vcc = vcc;
        newTranscriptionDTO.userId = userId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.SaveTranscription(newTranscriptionDTO);
        }).then(function (resultSaveTranscription) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: resultSaveTranscription });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
}
exports.AudioService = AudioService;
function SaveAudio(vcc, audio) {
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveAudio(vcc, audio.id, audio.section, audio.part, audio.processId, audio.number, audio.gender, audio.silenceTime, audio.durationTime, audio.category1, audio.category2, audio.category3, audio.description, audio.campaign, audio.agent, audio.type, audio.init));
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
function UpdateAudioInit(vcc, audio, fileNameAnalysisSemantic, fileNameAnalysisSemanticView) {
    var fileContent = '';
    return Promise.resolve()
        .then(function (result) {
        logger.info('[AudioService::UpdateAudioInit] UpdateAudioInit: ', audio);
        return file_manager_1.ReadFileContent(fileNameAnalysisSemantic, 'utf-8');
    }).catch(function (err) {
        return GetIdathaDocument(vcc, audio, fileNameAnalysisSemantic);
    })
        .then(function (result) {
        return file_manager_1.ReadFileContent(fileNameAnalysisSemantic, 'utf-8');
    })
        .then(function (resultReadFileContent) {
        fileContent = resultReadFileContent;
        logger.info('[AudioService::UpdateAudioInit] Received ReadFileContent\'s result: ' + JSON.stringify(resultReadFileContent));
        var result = JSON.parse(fileContent).words;
        audio.init = result.length > 0 ? result[0].speaker : null;
        return SaveAudio(vcc, audio);
    }).then(function (result) {
        var resultSortJson = SortJson(fileContent);
        fileContent = resultSortJson;
        return file_manager_1.WriteFileContent(fileNameAnalysisSemanticView, resultSortJson, 'utf-8');
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        logger.error('Error al ejecutar UpdateAudioInit: ' + errMsg);
        throw errMsg;
    });
}
function SortJson(jsonobj) {
    var outlength;
    var inlength;
    var i;
    var audioInit;
    var obj = {};
    obj.transcription = [];
    obj.summarySpeechData = {};
    obj.confidence = {};
    obj.init = '';
    var json = JSON.parse(jsonobj);
    audioInit = json.words[0].speaker;
    obj.init = audioInit;
    obj.nlp = {};
    outlength = json.out_sentences.length;
    inlength = json.in_sentences.length;
    if (outlength >= inlength) {
        for (i in json.out_sentences) {
            if (i < inlength && inlength > 0) {
                if (audioInit === 'out' && inlength > 0) {
                    obj.transcription.push({
                        out: json.out_sentences[i].text,
                        in: json.in_sentences[i] === 'undefined' ? '' : json.in_sentences[i].text
                    });
                }
                else if (audioInit === 'in' && inlength > 0) {
                    obj.transcription.push({
                        in: json.in_sentences[i].text,
                        out: json.out_sentences[i].text
                    });
                }
            }
            else {
                if (audioInit === 'out' && inlength >= 0) {
                    obj.transcription.push({
                        out: json.out_sentences[i] === undefined ? '' : json.out_sentences[i].text,
                        in: ''
                    });
                }
                else if (audioInit === 'in' && inlength >= 0) {
                    obj.transcription.push({
                        in: '',
                        out: json.out_sentences[i] === undefined ? '' : json.out_sentences[i].text
                    });
                }
            }
        }
    }
    else if (outlength <= inlength) {
        for (i in json.in_sentences) {
            if (i < outlength && outlength > 0) {
                if (audioInit === 'out' && outlength > 0) {
                    obj.transcription.push({
                        out: json.out_sentences[i].text,
                        in: json.in_sentences[i].text
                    });
                }
                else if (audioInit === 'in' && outlength > 0) {
                    obj.transcription.push({
                        in: json.in_sentences[i].text,
                        out: json.out_sentences[i].text
                    });
                }
            }
            else {
                if (audioInit === 'out' && inlength >= 0) {
                    obj.transcription.push({
                        out: '',
                        in: json.in_sentences[i] === undefined ? '' : json.in_sentences[i].text,
                    });
                }
                else if (audioInit === 'in' && inlength >= 0) {
                    obj.transcription.push({
                        in: json.in_sentences[i] === undefined ? '' : json.in_sentences[i].text,
                        out: ''
                    });
                }
            }
        }
    }
    obj.nlp.in = json.pnl_scores_in ? json.pnl_scores_in : 0;
    obj.nlp.out = json.pnl_scores_out ? json.pnl_scores_out : 0;
    var dataTotalIn = {
        duration: 0,
        silence: 0
    }, dataTotalOut = {
        duration: 0,
        silence: 0
    }, data = json.words, len = data.length, word, inDuration = 0, inSilence = 0, outDuration = 0, outSilence = 0;
    for (i = 0; i < len; i++) {
        word = data[i];
        if (word.speaker === 'in') {
            inDuration += word.duration;
            inSilence += word.silence;
        }
        else {
            outDuration += word.duration;
            outSilence += word.silence;
        }
    }
    dataTotalIn.duration = inDuration.toFixed(2) * 1;
    dataTotalIn.silence = inSilence.toFixed(2) * 1;
    dataTotalOut.duration = outDuration.toFixed(2) * 1;
    dataTotalOut.silence = outSilence.toFixed(2) * 1;
    obj.summarySpeechData = {
        dataTotalIn: dataTotalIn,
        dataTotalOut: dataTotalOut
    };
    obj.duration = json.duration;
    obj.words = json.words;
    obj.topics = json.topics;
    obj.confidence.in_confidence = json.in_confidence ? json.in_confidence : 0;
    obj.confidence.out_confidence = json.out_confidence ? json.out_confidence : 0;
    return JSON.stringify(obj);
}
function ExportAudioTranscription(vcc, audio, fileNameAnalysisSemantic, fileNameAnalysisSemanticView) {
    return Promise.resolve()
        .then(function (result) {
        return file_manager_1.CreateFolder(config_1.SEMANCTIC_ANALYSIS_FOLDER + '/' + vcc);
    }).then(function (result) {
        return file_manager_1.CreateFolder(config_1.SEMANCTIC_ANALYSIS_VIEW_FOLDER + '/' + vcc);
    }).then(function () {
        return file_manager_1.ExistInFileSystem(fileNameAnalysisSemanticView);
    }).then(function (resultExistInFileSystem) {
        if (!resultExistInFileSystem) {
            return UpdateAudioInit(vcc, audio, fileNameAnalysisSemantic, fileNameAnalysisSemanticView);
        }
        return Promise.resolve(true);
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        logger.error('[AudioService::ExportAudioTranscription] Getting error:', err, 'with vcc:', vcc, 'with audio:', audio, 'fileNameAnalysisSemantic:', fileNameAnalysisSemantic, 'fileNameAnalysisSemanticView:', fileNameAnalysisSemanticView);
        throw errMsg;
    });
}
function GetValidAudios(vcc, process, userId) {
    return file_manager_1.ReadFileContent(config_1.PATH_FOLDER_PROCESS_AUDIO + '/' + process.name + '/dataprocess.csv', 'utf-8', true)
        .then(function (resultReadFileContent) {
        return csvtojson({
            delimiter: 'auto',
            noheader: false,
            checkType: true,
            checkColumn: true,
            includeColumns: /(^ID$|^Number$|^Gender$|^SilenceTime$|^DurationTime$|^Category1$|^Category2$|^Category3$|^Description$|^Campaign$|^Agent$|^Type$)/
        }).fromString(resultReadFileContent);
    }).then(function (resultConvertCsvToJson) {
        var arrayHeader;
        var validateHeader;
        var resultConvertCsvToJsonFilter = [];
        if (resultConvertCsvToJson.length > 0) {
            arrayHeader = Object.keys(resultConvertCsvToJson[0]);
            validateHeader = arrayHeader.includes('ID') && arrayHeader.includes('SilenceTime') && arrayHeader.includes('DurationTime') ? true : false;
            if (validateHeader) {
                for (var _i = 0, resultConvertCsvToJson_1 = resultConvertCsvToJson; _i < resultConvertCsvToJson_1.length; _i++) {
                    var row = resultConvertCsvToJson_1[_i];
                    if (typeof row.SilenceTime === 'number' && typeof row.DurationTime === 'number' && row.ID && row.ID.length <= 100) {
                        resultConvertCsvToJsonFilter.push(row);
                    }
                }
                return resultConvertCsvToJsonFilter;
            }
            else {
                return null;
            }
        }
        else {
            return null;
        }
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        logger.error('Error al ejecutar GetBulkAudio: ' + errMsg);
    });
}
function GetBulkAudiosSummary(vcc, process, userId) {
    return file_manager_1.ReadFileContent(config_1.PATH_FOLDER_PROCESS_AUDIO + '/' + process.name + '/' + 'dataprocess' + '.csv', 'utf-8')
        .then(function (resultReadFileContent) {
        return csvtojson().fromString(resultReadFileContent);
    }).then(function (resultConvertCsvToJson) {
        var summary = alasql('SELECT Category1, Category2, Category3, Description, COUNT(*) AS Total FROM ? WHERE Category1 != \'\' OR Description != \'\' GROUP BY Category1, Category2, Category3, Description', [resultConvertCsvToJson]);
        var objSummary = [];
        for (var _i = 0, summary_1 = summary; _i < summary_1.length; _i++) {
            var row = summary_1[_i];
            var row2 = row;
            if (row.Category1 === '') {
                row2.Category1 = row.Description;
                row2.Description = '';
            }
            else if (row.Category1 !== '' && row.Category2 === '') {
                row2.Category2 = row.Description;
                row2.Description = '';
            }
            else if (row.Category2 !== '' && row.Category3 === '') {
                row2.Category3 = row.Description;
                row2.Description = '';
            }
            objSummary.push(row2);
        }
        return mssql_service_1.MSSql.BulkAudioSummaryTemp(vcc, process, objSummary, userId);
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        logger.error('Error al ejecutar GetBulkAudioSummary: ' + errMsg);
    });
}
function ExternalRequest(url) {
    var options = {
        method: 'GET',
        url: url,
        encoding: null,
        timeout: 30000
    };
    logger.info('ExternalRequest: ', options);
    return request(options)
        .then(function (response) {
        logger.info("ExternalRequest OK");
        return Promise.resolve(response);
    })
        .catch(function (err) {
        logger.info("ExternalRequest KO");
        return Promise.reject(err);
    });
}
exports.ExternalRequest = ExternalRequest;
function getTimes(startTime, endTime) {
    var startHour, startMinute, endHour, endMinute;
    if (startTime && startTime.length > 0) {
        var arrayStartTime = startTime.split(':');
        if (arrayStartTime && arrayStartTime.length > 0) {
            startHour = arrayStartTime[0];
            startMinute = arrayStartTime[1];
        }
    }
    else {
        startHour = 0;
        startMinute = 0;
    }
    if (endTime && endTime.length > 0) {
        var arrayEndTime = endTime.split(':');
        if (arrayEndTime && arrayEndTime.length > 0) {
            endHour = arrayEndTime[0];
            endMinute = arrayEndTime[1];
        }
    }
    else {
        endHour = 23;
        endMinute = 59;
    }
    return {
        startTime: {
            hour: startHour,
            minute: startMinute,
        },
        endTime: {
            hour: endHour,
            minute: endMinute,
        }
    };
}
function GetIdathaDocument(vcc, audio, fileName) {
    var idatha = {
        user: config_1.IDATHA_USER,
        password: config_1.IDATHA_PASSWORD,
        ip: config_1.IDATHA_IP,
        httpPort: config_1.IDATHA_HTTP_PORT,
        sshPort: config_1.IDATHA_SSH_PORT,
    };
    var userName = '';
    var password = '';
    var accessToken = '';
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        return mssql_service_1.MSSql.GetUserAccessTokenByRealm(vcc, vcc);
    })
        .then(function (resultUserAccessTokenByRealm) {
        if (resultUserAccessTokenByRealm.accessToken) {
            accessToken = resultUserAccessTokenByRealm.accessToken;
            var currentDate = new Date(), createdDate = new Date(resultUserAccessTokenByRealm.createdDate), diffDays = Math.round(Math.abs((currentDate.getTime() - createdDate.getTime()) / (24 * 60 * 60 * 1000)));
            if (diffDays >= 7) {
                return idatha_service_1.IdathaLogin(idatha, vcc);
            }
            else {
                accessToken = resultUserAccessTokenByRealm.accessToken;
                return accessToken;
            }
        }
        else {
            return resultUserAccessTokenByRealm;
        }
    })
        .then(function (result) {
        return IdathaDocumentWords(idatha, accessToken, audio.id, audio.section, audio.part);
    }).then(function (resultIdathaDocumentWords) {
        var regex = /\bNaN\b/g;
        var data;
        data = JSON.stringify(resultIdathaDocumentWords.response);
        if (regex.test(resultIdathaDocumentWords.response))
            data = resultIdathaDocumentWords.response.replace(regex, 'null');
        if (resultIdathaDocumentWords.response)
            return file_manager_1.WriteFileContent(fileName, data, 'utf-8');
        return true;
    });
}
exports.GetIdathaDocument = GetIdathaDocument;
function IdathaDocumentWords(idatha, accessToken, audioId, section, part) {
    var url = 'http://';
    url += idatha.ip;
    url += ':';
    url += idatha.httpPort;
    url += '/api/query/document/';
    url += audioId;
    if (section !== '' && part !== '0') {
        url += '_';
        url += section;
        url += '_';
        url += part;
    }
    var options = {
        method: 'GET',
        uri: url,
        headers: {
            'Authorization': 'Bearer ' + accessToken
        },
        json: true,
        resolveWithFullResponse: false
    };
    logger.info('IdathaDocumentWords - options: ', options);
    return request(options)
        .then(function (response) {
        var content = { response: response, type: 'document' };
        return Promise.resolve(content);
    }).catch(function (err) {
        options = {
            method: 'GET',
            uri: 'http://' + idatha.ip + ':' + idatha.httpPort + '/api/query/document/' + audioId,
            headers: {
                'Authorization': 'Bearer ' + accessToken
            },
            json: true,
            resolveWithFullResponse: false
        };
        logger.info('IdathaDocumentWords - options - catch: ', options);
        return request(options)
            .then(function (response) {
            var content = { response: response, type: 'document' };
            return Promise.resolve(content);
        }).catch(function (err) {
            logger.error('Ha sucedido el error: ' + err.message);
            return Promise.reject(err);
        });
    });
}
exports.IdathaDocumentWords = IdathaDocumentWords;
function GetAudioFileFromFtp(vcc, audioId, environment) {
    var environmentConfig = {
        host: environment.apiHost,
        port: environment.apiPort,
        user: environment.apiUser,
        password: environment.apiPassword,
        connTimeout: config_1.FTP_CONNECTION_TIMEOUT,
        pasvTimeout: config_1.FTP_CONNECTION_TIMEOUT
    };
    var genericResult = {};
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        return new Promise(function (resolve, reject) {
            var clientFtp = new Client();
            clientFtp.connect(environmentConfig);
            if (clientFtp) {
                clientFtp.on('greeting', function (message) {
                    logger.info('[AudioService::GetAudioFileFromFtp] Greeting event message:', message);
                });
                clientFtp.on('ready', function () {
                    clientFtp.list(environment.ftpDirectory, function (err, list) {
                        if (err)
                            return resolve(err.message);
                        var fileNameToFind = audioId;
                        var regex = new RegExp(fileNameToFind + '\.(mp3|wav)$');
                        var foundFiles = list.filter(function (x) { return x.name.match(regex); });
                        if (foundFiles && foundFiles.length && foundFiles.length > 0) {
                            var fileName_1 = foundFiles[0].name;
                            var ftpFile = '';
                            ftpFile += environment.ftpDirectory;
                            ftpFile += '/';
                            ftpFile += fileName_1;
                            logger.info('[AudioService::GetAudioFileFromFtp] FtpFilePath: ', ftpFile);
                            clientFtp.get(ftpFile, function (err, stream) {
                                if (err) {
                                    clientFtp.end();
                                    logger.error('[AudioService::GetAudioFileFromFtp] ERROR_AUDIO_FILE_GET_FTP: ', err);
                                    genericResult.status = false;
                                    genericResult.extraContent = 'SERVER_ERROR_AUDIO_FILE_GET_FTP';
                                    return reject(genericResult);
                                }
                                else {
                                    genericResult.status = true;
                                    var bufs_1 = [];
                                    stream.once('close', function () {
                                        clientFtp.end();
                                        genericResult.extraContent = Buffer.concat(bufs_1);
                                        resolve(fileName_1);
                                    });
                                    stream.on('data', function (d) {
                                        bufs_1.push(d);
                                    });
                                }
                            });
                        }
                        else {
                            clientFtp.end();
                            genericResult.status = false;
                            genericResult.error = 'SERVER_ERROR_AUDIO_FILE_NOT_FOUND';
                            return resolve(genericResult);
                        }
                    });
                });
                clientFtp.on('end', function () {
                    var eventMessage = 'FTP_CONNECTION_ENDED';
                    logger.warn('[AudioTransferImplementation::GetFTPAudioFile] getting eventMessage:', eventMessage);
                    return resolve(eventMessage);
                });
                clientFtp.on('close', function (hadError) {
                    var eventMessage = hadError ? 'FTP_CONNECTION_CLOSE_DUE_ERROR' : 'FTP_CONNECTION_CLOSE';
                    logger.warn('[AudioTransferImplementation::GetFTPAudioFile] hadError:', hadError);
                    return resolve(eventMessage);
                });
                clientFtp.on('error', function (err) {
                    logger.error('[GetFTPAudioFile] NOT_FTP_CLIENT', err);
                    return resolve('NOT_FTP_CLIENT');
                });
            }
            else {
                logger.error('[AudioService::GetAudioFileFromFtp] Can not create FTP Client');
                return reject('SERVER_ERROR_CANNOT_CREATE_FTP_CLIENT');
            }
        });
    }).then(function (result) {
        return Promise.resolve({
            audioFile: result,
            buffer: genericResult.extraContent
        });
    }).catch(function (error) {
        return Promise.reject(error);
    });
}
function GetAudioFileFromSftp(vcc, audioId, environment) {
    var environmentConfig = {
        host: environment.apiHost,
        port: environment.apiPort,
        username: environment.apiUser,
        password: environment.apiPassword,
        connTimeout: config_1.FTP_CONNECTION_TIMEOUT,
        pasvTimeout: config_1.FTP_CONNECTION_TIMEOUT
    };
    var audioFile = null;
    var genericResult = {};
    var ssh2 = new SSH2Promise(environmentConfig);
    var sftp;
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        if (ssh2) {
            sftp = ssh2.sftp();
            return sftp.readdir(environment.ftpDirectory)
                .then(function (list) {
                logger.info('[AudioService::GetAudioFileFromSftp] Folder items: ' + list.length);
                if (list && list.length) {
                    var fileNameToFind = audioId;
                    var regex_1 = new RegExp(fileNameToFind + '\.(mp3|wav)$');
                    var foundFiles = list.filter(function (x) {
                        return x.filename.match(regex_1);
                    });
                    if (foundFiles && foundFiles.length && foundFiles.length > 0) {
                        logger.info('[AudioService::GetAudioFileFromSftp] Found item from SFTP');
                        audioFile = foundFiles[0].filename;
                        var sftpFile = '';
                        sftpFile += environment.ftpDirectory;
                        sftpFile += '/';
                        sftpFile += audioFile;
                        return sftp.createReadStream(sftpFile);
                    }
                    else {
                        logger.error('[AudioService::GetAudioFileFromSftp] SERVER_ERROR_AUDIO_ID_NOT_FOUND');
                        throw 'SERVER_ERROR_AUDIO_ID_NOT_FOUND';
                    }
                }
                else {
                    logger.error('[AudioService::GetAudioFileFromSftp] SERVER_ERROR_AUDIO_FILES_NOT_FOUND');
                    throw 'SERVER_ERROR_AUDIO_FILES_NOT_FOUND';
                }
            });
        }
        else {
            logger.error('[AudioService::GetAudioFileFromSftp] SERVER_ERROR_NOT_SFTP_CLIENT');
            throw 'SERVER_ERROR_NOT_SFTP_CLIENT';
        }
    }).then(function (stream) {
        if (stream) {
            genericResult.status = true;
            var bufs_2 = [];
            return new Promise(function (resolve, reject) {
                stream.once('close', function () {
                    sftp.close();
                    genericResult.extraContent = Buffer.concat(bufs_2);
                    return resolve(genericResult.extraContent);
                });
                stream.on('data', function (d) {
                    bufs_2.push(d);
                });
            });
        }
        else {
            logger.error('[AudioService::GetAudioFileFromSftp] SERVER_ERROR_GET_AUDIO_STREAM_FAILED');
            throw 'SERVER_ERROR_GET_AUDIO_STREAM_FAILED';
        }
    }).then(function (result) {
        if (genericResult.status === true) {
            return Promise.resolve(genericResult.extraContent);
        }
        else {
            logger.error('[AudioService::GetAudioFileFromSftp] SERVER_ERROR_GET_AUDIO_FROM_SFTP_FAILED');
            throw 'SERVER_ERROR_GET_AUDIO_FROM_SFTP_FAILED';
        }
    }).catch(function (err) {
        genericResult.status = false;
        genericResult.error = err;
        logger.error('[AudioService::GetAudioFileFromSftp] General error: ', err);
        return Promise.reject({
            audioFile: audioFile,
            buffer: genericResult.error
        });
    });
}
function GetAudioFileFromAllegro(vcc, audioId, section, part, fileName, ext, target) {
    if (ext === null)
        ext = '.wav';
    if (target === null)
        target = 'LOCAL';
    var fileNameAudio = null;
    if (fileName === null) {
        fileNameAudio = audioId;
        fileNameAudio += '_';
        fileNameAudio += section;
        fileNameAudio += '_';
        fileNameAudio += part;
        fileNameAudio += ext;
    }
    if (fileNameAudio !== null) {
        var filePath = null;
        if (target === 'LOCAL') {
            filePath = part !== null && !isNaN(part) && part > 0 ? config_1.PATH_FOLDER_TEMP_AUDIO : config_1.PATH_FOLDER_PROCESS_AUDIO;
            filePath += '/';
            filePath += vcc;
            filePath += '/';
        }
        else if (target === 'REMOTE') {
            filePath = 'http://';
            filePath += integration_config_1.IP_MW;
            filePath += ':';
            filePath += integration_config_1.PORT_MW;
            filePath += '/inconcert/apps/mediaserver/action/download/repository/';
            filePath += audioId;
            filePath += '/';
            filePath += section;
            filePath += '/';
            filePath += part;
            filePath += '/';
        }
        fileNameAudio = filePath + fileNameAudio;
    }
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        switch (target) {
            case 'LOCAL': return file_manager_1.BufferFile(fileNameAudio);
            case 'REMOTE': return ExternalRequest(fileNameAudio);
            default: throw 'SERVER_ERROR_INVALID_TARGET';
        }
    }).catch(function (error) {
        logger.error('[AudioService::GetAudioFileFromAllegro] Tried: ', fileNameAudio);
        if (ext === '.wav' && target === 'LOCAL' && (fileName === null || fileName.includes('.mp3')))
            ext = '.mp3';
        else if (ext === '.mp3' && target === 'LOCAL' && (fileName === null || fileName.includes('.wav'))) {
            ext = '.wav';
            target = 'REMOTE';
        }
        else if (ext === '.wav' && target === 'REMOTE' && (fileName === null || fileName.includes('.mp3')))
            ext = '.mp3';
        else if (ext === '.mp3' && target === 'REMOTE')
            throw 'SERVER_ERROR_AUDIO_FILE_NOT_FOUND';
        fileNameAudio = null;
        return GetAudioFileFromAllegro(vcc, audioId, section, part, null, ext, target);
    }).then(function (result) {
        if (!result)
            'SERVER_ERROR_AUDIO_FILE_NOT_FOUND';
        var res = {};
        if (fileNameAudio) {
            logger.info('[AudioService::GetAudioFileFromAllegro] Found on: ', fileNameAudio);
            res = {
                audioFile: fileNameAudio,
                buffer: result
            };
        }
        else {
            res = result;
        }
        return Promise.resolve(res);
    }).catch(function (error) {
        return Promise.reject(error);
    });
}
