"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var evaluation_result_model_1 = require("../models/evaluation-result.model");
var utilities_1 = require("../shared/utilities");
var logger = log4js.getLogger('ServerScripts');
function EvaluationService(app) {
    app.post('/api/evaluation/get_evaluation_results/', function (req, res, next) {
        logger.info('[EvaluationService::get_evaluation_results] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var userId = res.locals.userId;
        var evaluationPlanId = req.body.evaluationPlanId || null;
        var interactionId = req.body.interactionId || null;
        var auditorId = req.body.auditorId || userId || null;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetEvaluationResults(vcc, evaluationPlanId, interactionId, auditorId);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[EvaluationService::get_evaluation_results] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/save_evaluationresult', function (req, res, next) {
        logger.info('[EvaluationService::save_evaluationresult] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var userId = res.locals.userId;
        var evaluations = (req.body || {});
        SaveEvaluationResults(instance, userId, evaluations)
            .then(function (result) {
            res.json({ status: true, description: (result ? 'OK' : 'Evaluation Results save failed'), data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/evaluation/get_interaction_evaluation/:interaction?', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var interaction = req.params.interaction || '';
        var interactionType = req.query.interactionType || '';
        var evaluationDateFrom = req.query.evaluationDateFrom || '1980-01-01';
        var evaluationDateTo = req.query.evaluationDateTo || '2050-01-01';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetEvaluationsOfInteraction(instance, interaction, interactionType, evaluationDateFrom, evaluationDateTo);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/get_evaluation_agent/', function (req, res, next) {
        logger.info('[EvaluationService::get_evaluation_agent] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var idAgent = res.locals.userId;
        logger.info('[EvaluationService::get_evaluation_agent] Filling the filter\'s object');
        var params = new evaluation_result_model_1.EvaluationResultFilters;
        params.dateIni = req.body.dateIni || '';
        params.dateFin = req.body.dateFin || '';
        params.campaigns = req.body.campaigns || [];
        params.evaluationPlans = req.body.evaluationPlans || [];
        params.origins = req.body.origins || [];
        params.minDuration = req.body.minDuration || null;
        params.maxDuration = req.body.maxDuration || null;
        params.evaluationTypes = req.body.evaluationTypes || [];
        params.tags = req.body.tags || [];
        params.hasVideoOptions = req.body.hasVideoOptions || [];
        params.results = req.body.results ? req.body.results.map(function (r) {
            var dispositionCode = null;
            if (typeof r === 'string') {
                dispositionCode = r;
            }
            else {
                dispositionCode = r.campaignId;
                if (r.description)
                    dispositionCode += '_' + r.description;
                if (r.description2)
                    dispositionCode += '_' + r.description2;
                if (r.description3)
                    dispositionCode += '_' + r.description3;
                if (r.description4)
                    dispositionCode += '_' + r.description4;
            }
            return dispositionCode;
        }) : [];
        params.hourIni = req.body.hourIni || '';
        params.hourFin = req.body.hourFin || '';
        params.multimediaFormats = ['mp3', 'wav'];
        if (params.hasVideoOptions.includes(true)) {
            params.multimediaFormats.push('flv');
        }
        params.origins = params.origins.map(function (o) {
            if (o == "E")
                return "IN";
            else if (o == "S")
                return "OUT";
            else
                return null;
        });
        params.interactionIds = req.body.interactionIds || [];
        logger.debug('[EvaluationService::get_evaluation_agent] Filter\'s object: ', JSON.stringify(params));
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[EvaluationService::get_evaluation_agent] Calling GetEvaluations SP for VCC|idAgent: ' + instance + '|' + idAgent);
            return mssql_service_1.MSSql.GetEvaluations(instance, null, idAgent, params.campaigns.toString(), params.evaluationPlans.toString(), params.dateIni, params.dateFin, params.hourIni, params.hourFin, params.minDuration, params.maxDuration, params.origins.toString(), params.results.toString(), params.evaluationTypes.toString(), params.tags.toString(), params.multimediaFormats.toString(), params.interactionIds.toString());
        })
            .then(function (result) {
            logger.debug('[EvaluationService::get_evaluation_agent] Received result from GetEvaluations SP: ' + JSON.stringify(result));
            if (res.headersSent)
                return;
            logger.info('[EvaluationService::get_evaluation_agent] Sending results: ' + result.length);
            res.json({ status: true, description: 'OK', data: result });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationService::get_evaluation_agent] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/get_evaluation_auditor/', function (req, res, next) {
        logger.info('[EvaluationService::get_evaluation_auditor] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var idAuditor = res.locals.userId;
        logger.info('[EvaluationService::get_evaluation_auditor] Filling the filter\'s object');
        var params = new evaluation_result_model_1.EvaluationResultFilters;
        params.dateIni = req.body.dateIni || '';
        params.dateFin = req.body.dateFin || '';
        params.campaigns = req.body.campaigns || [];
        params.evaluationPlans = req.body.evaluationPlans || [];
        params.agents = req.body.agents || [];
        params.origins = req.body.origins || [];
        params.minDuration = req.body.minDuration || null;
        params.maxDuration = req.body.maxDuration || null;
        params.evaluationTypes = req.body.evaluationTypes || [];
        params.tags = req.body.tags || [];
        params.hasVideoOptions = req.body.hasVideoOptions || [];
        params.results = req.body.results ? req.body.results.map(function (r) {
            var dispositionCode = null;
            if (typeof r === 'string') {
                dispositionCode = r;
            }
            else {
                dispositionCode = r.campaignId;
                if (r.description)
                    dispositionCode += '_' + r.description;
                if (r.description2)
                    dispositionCode += '_' + r.description2;
                if (r.description3)
                    dispositionCode += '_' + r.description3;
                if (r.description4)
                    dispositionCode += '_' + r.description4;
            }
            return dispositionCode;
        }) : [];
        params.hourIni = req.body.hourIni || '';
        params.hourFin = req.body.hourFin || '';
        params.multimediaFormats = ['mp3', 'wav'];
        if (params.hasVideoOptions.includes(true)) {
            params.multimediaFormats.push('flv');
        }
        params.origins = params.origins.map(function (o) {
            if (o == "E")
                return "IN";
            else if (o == "S")
                return "OUT";
            else
                return null;
        });
        params.interactionIds = req.body.interactionIds || [];
        logger.debug('[EvaluationService::get_evaluation_auditor] Filter\'s object: ', JSON.stringify(params));
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[EvaluationService::get_evaluation_auditor] Calling GetEvaluations SP for VCC|IdAuditor: ' + instance + '|' + idAuditor);
            return mssql_service_1.MSSql.GetEvaluations(instance, idAuditor, params.agents.toString(), params.campaigns.toString(), params.evaluationPlans.toString(), params.dateIni, params.dateFin, params.hourIni, params.hourFin, params.minDuration, params.maxDuration, params.origins.toString(), params.results.toString(), params.evaluationTypes.toString(), params.tags.toString(), params.multimediaFormats.toString(), params.interactionIds.toString());
        })
            .then(function (result) {
            logger.debug('[EvaluationService::get_evaluation_auditor] Received result from GetEvaluations SP: ' + JSON.stringify(result));
            if (res.headersSent)
                return;
            logger.info('[EvaluationService::get_evaluation_auditor] Sending results: ' + result.length);
            res.json({ status: true, description: 'OK', data: result });
        })
            .catch(function (err) {
            (function (err) {
                var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
                var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error requesting');
                logger.error('[EvaluationService::get_evaluation_auditor] Error on \'' + req.path + '\': ' + errorMsg);
                if (res.headersSent)
                    return;
                res.json({ status: false, description: msg, error: errorMsg });
            });
        });
    });
    app.post('/api/evaluation/get_evaluation_admin/', function (req, res, next) {
        logger.info('[EvaluationService::get_evaluation_admin] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        logger.info('[EvaluationService::get_evaluation_admin] Filling the filter\'s object');
        var params = new evaluation_result_model_1.EvaluationResultFilters;
        params.dateIni = req.body.dateIni || '';
        params.dateFin = req.body.dateFin || '';
        params.campaigns = req.body.campaigns || [];
        params.evaluationPlans = req.body.evaluationPlans || [];
        params.auditors = req.body.auditors || [];
        params.agents = req.body.agents || [];
        params.origins = req.body.origins || [];
        params.minDuration = req.body.minDuration || null;
        params.maxDuration = req.body.maxDuration || null;
        params.evaluationTypes = req.body.evaluationTypes || [];
        params.tags = req.body.tags || [];
        params.hasVideoOptions = req.body.hasVideoOptions || [];
        params.results = req.body.results ? req.body.results.map(function (r) {
            var dispositionCode = null;
            if (typeof r === 'string') {
                dispositionCode = r;
            }
            else {
                dispositionCode = r.campaignId;
                if (r.description)
                    dispositionCode += '_' + r.description;
                if (r.description2)
                    dispositionCode += '_' + r.description2;
                if (r.description3)
                    dispositionCode += '_' + r.description3;
                if (r.description4)
                    dispositionCode += '_' + r.description4;
            }
            return dispositionCode;
        }) : [];
        params.hourIni = req.body.hourIni || '';
        params.hourFin = req.body.hourFin || '';
        params.multimediaFormats = ['mp3', 'wav'];
        if (params.hasVideoOptions.includes(true)) {
            params.multimediaFormats.push('flv');
        }
        params.origins = params.origins.map(function (o) {
            if (o == "E")
                return "IN";
            else if (o == "S")
                return "OUT";
            else
                return null;
        });
        params.interactionIds = req.body.interactionIds || [];
        logger.debug('[EvaluationService::get_evaluation_admin] Filter\'s object: ', JSON.stringify(params));
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[EvaluationService::get_evaluation_admin] Calling GetEvaluations SP for VCC|IdAuditor: ' + instance);
            return mssql_service_1.MSSql.GetEvaluations(instance, params.auditors.toString(), params.agents.toString(), params.campaigns.toString(), params.evaluationPlans.toString(), params.dateIni, params.dateFin, params.hourIni, params.hourFin, params.minDuration, params.maxDuration, params.origins.toString(), params.results.toString(), params.evaluationTypes.toString(), params.tags.toString(), params.multimediaFormats.toString(), params.interactionIds.toString());
        })
            .then(function (result) {
            logger.debug('[EvaluationService::get_evaluation_admin] Received result from GetEvaluations SP: ' + JSON.stringify(result));
            if (res.headersSent)
                return;
            logger.info('[EvaluationService::get_evaluation_admin] Sending results: ' + result.length);
            res.json({ status: true, description: 'OK', data: result });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/get_evaluation_results_from_auditor_evaluation_plan/:evaluationPlanId', function (req, res, next) {
        logger.info('[EvaluationService::get_evaluation_results_from_auditor_evaluation_plan] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var auditorId = res.locals.userId;
        logger.info('[EvaluationService::get_evaluation_results_from_auditor_evaluation_plan] Filling the filter\'s object');
        var params = new evaluation_result_model_1.EvaluationResultInteractionFilters;
        params.evaluationPlanId = req.params.evaluationPlanId || '';
        params.interactionId = req.body.interactionId || '';
        params.auditorId = req.body.auditorId || '';
        params.agentId = req.body.agentId || '';
        params.interactionType = req.body.interactionType || '';
        var results = {};
        logger.debug('[EvaluationService::get_evaluation_results_from_auditor_evaluation_plan] Filter\'s object: ', JSON.stringify(params));
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[EvaluationService::get_evaluation_results_from_auditor_evaluation_plan] Calling GetEvaluationResultsFromAuditorEvaluationPlan SP for VCC|IdEP: ' + instance + '|' + params.evaluationPlanId);
            return mssql_service_1.MSSql.GetEvaluationResultsFromAuditorEvaluationPlan(instance, params.evaluationPlanId, params.interactionId, params.interactionType, params.auditorId, params.agentId);
        })
            .then(function (result) {
            results.evaluationResultData = result;
            logger.info('[EvaluationService::get_evaluation_results_from_auditor_evaluation_plan] Calling GetEvaluationCustomFieldResultsFromAuditorEvaluationPlan SP for VCC|IdEP: ' + instance + '|' + params.evaluationPlanId);
            return mssql_service_1.MSSql.GetEvaluationCustomFieldResultsFromAuditorEvaluationPlan(instance, params.evaluationPlanId, params.interactionId, params.auditorId, params.agentId);
        })
            .then(function (result) {
            results.evaluationCustomFieldResultData = result;
            return mssql_service_1.MSSql.GetEvaluationFormatAttachedSignatures(instance, results.evaluationResultData[0].EvaluationFormatId);
        })
            .then(function (result) {
            results.attachedSignatures = result;
            if (res.headersSent)
                return;
            logger.info('[EvaluationService::get_evaluation_results_from_auditor_evaluation_plan] Received result from GetEvaluationResultsFromAuditorEvaluationPlan SP');
            res.json({ status: true, description: 'OK', data: results });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error requesting');
            logger.error('[EvaluationService::get_evaluation_results_from_auditor_evaluation_plan] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.get('/api/evaluation/get_evaluation_reports/:evformatid?', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var idEvaluationFormat = req.params.evformatid || '';
        var idsCampanias = req.query.campaigns || null, idAgente = req.query.agent || null, fechaDesde = req.query.dateFrom || null, fechaHasta = req.query.dateTo || null, idAuditor = req.query.auditor || null, planAuditoria = req.query.evaluationPlanId || null;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.ReportAuditsByAgent(instance, idEvaluationFormat, idsCampanias, idAgente, fechaDesde, fechaHasta, idAuditor, planAuditoria);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
}
exports.EvaluationService = EvaluationService;
function SaveEvaluationResults(instance, userId, interactionEvaluationPlanResult) {
    logger.info('[EvaluationService::SaveEvaluationResults] interactionEvaluationPlanResult: ', interactionEvaluationPlanResult);
    return new Promise(function (resolve, reject) {
        var nowUtc = moment().utc().toDate();
        var evaluationResultPromises = [];
        var evaluationCustomFieldResultPromises = [];
        var ok;
        var evaluationPlanId;
        var interactionId;
        var interactionType;
        var summarizedScore = 0;
        if (interactionEvaluationPlanResult && interactionEvaluationPlanResult.evaluationResult.length > 0) {
            ok = true;
            evaluationPlanId = interactionEvaluationPlanResult.evaluationResult[0].idEvaluationPlan;
            interactionId = interactionEvaluationPlanResult.evaluationResult[0].idInteraction;
            interactionType = interactionEvaluationPlanResult.evaluationResult[0].interactionType;
            interactionEvaluationPlanResult.evaluationResult.forEach(function (newEvaluationResult) {
                if (!newEvaluationResult.id) {
                    newEvaluationResult.id = utilities_1.guid();
                    newEvaluationResult.createdDate = nowUtc;
                    newEvaluationResult.createdByUserId = userId;
                }
                else {
                    newEvaluationResult.createdDate = null;
                    newEvaluationResult.createdByUserId = null;
                }
                newEvaluationResult.lastModifiedDate = nowUtc;
                newEvaluationResult.lastModifiedByUserId = userId;
                newEvaluationResult.idAuditor = userId;
                newEvaluationResult.evaluationDate = nowUtc;
                logger.debug('[EvaluationService::SaveEvaluationResults] Guardando resultado evaluacion:' + JSON.stringify(newEvaluationResult));
                evaluationResultPromises.push(promises_shared_1.DummyPromise()
                    .then(function (result) {
                    return mssql_service_1.MSSql.SaveEvaluationResult(instance, newEvaluationResult.id, newEvaluationResult.idCampaign, newEvaluationResult.idInteraction, newEvaluationResult.idAgent, newEvaluationResult.idEvaluationFormat, newEvaluationResult.minQualificationAcceptable, newEvaluationResult.idGroup, newEvaluationResult.idSubGroup, newEvaluationResult.idAbility, newEvaluationResult.qualification, newEvaluationResult.idAuditor, newEvaluationResult.evaluationDate, newEvaluationResult.maxAbilityScore, newEvaluationResult.idEvaluationPlan, newEvaluationResult.comments, newEvaluationResult.notApplies, newEvaluationResult.scoreDescription, newEvaluationResult.campaignDescription, newEvaluationResult.agentDescription, new Date(newEvaluationResult.interactionDateini), new Date(newEvaluationResult.interactionDatefin), newEvaluationResult.interactionAddress, newEvaluationResult.interactionType, newEvaluationResult.abilityOrder, newEvaluationResult.createdDate, newEvaluationResult.createdByUserId, newEvaluationResult.lastModifiedDate, newEvaluationResult.lastModifiedByUserId, newEvaluationResult.conditionalSelected);
                })
                    .then(function (result) {
                    logger.debug('[EvaluationService::SaveEvaluationResults] Procesado resultado linea ' + newEvaluationResult.id);
                    return result;
                })
                    .catch(function (err) {
                    ok = false;
                    reject(err);
                }));
            });
        }
        if (interactionEvaluationPlanResult && interactionEvaluationPlanResult.evaluationCustomFieldResults.length > 0) {
            interactionEvaluationPlanResult.evaluationCustomFieldResults.map(function (ecfr) {
                ecfr.results.map(function (r) {
                    evaluationCustomFieldResultPromises.push(mssql_service_1.MSSql.SaveEvaluationCustomFieldResult(instance, ecfr.evaluationPlanId, ecfr.interactionId, userId, ecfr.agentId, ecfr.customFieldId, r, userId, userId));
                });
            });
        }
        Promise.all(evaluationResultPromises)
            .then(function (data) {
            if (ok) {
                return Promise.all(evaluationCustomFieldResultPromises);
            }
            else
                return Promise.resolve([true]);
        })
            .then(function (data) {
            return mssql_service_1.MSSql.SaveInteractionEvaluationPlanResult(instance, evaluationPlanId, userId, interactionId, interactionType, interactionEvaluationPlanResult.interactionScore, interactionEvaluationPlanResult.interactionAutomaticScore, interactionEvaluationPlanResult.interactionManualScore);
        })
            .then(function (data) {
            return mssql_service_1.MSSql.DeleteAuditorEvaluationPlanByInteraction(instance, evaluationPlanId, interactionId, interactionType);
        })
            .then(function (data) {
            return mssql_service_1.MSSql.SaveEvaluationPlanCompleted(instance, evaluationPlanId);
        })
            .then(function (data) {
            if (ok)
                resolve(data);
            else
                reject(data);
        }).catch(function (err) {
            reject(err);
        });
    });
}
