"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var cognitive_analysis_implementation_1 = require("../implementation/cognitive-analysis.implementation");
var nlu_service_1 = require("./nlu.service");
var logger = log4js.getLogger('ServerScripts');
function UtteranceService(app) {
    var cognitiveService;
    app.get('/api/utterance/get_utterances/:app_id/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('[UtteranceService::get_utterances] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        var app_id = req.params.app_id || '';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!app_id)
                throw 'MISSING_NLU_INPUT_PARAMETER';
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, app_id);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_NLU_NOT_FOUND';
            if (result.isSystemApp) {
                return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
            }
            else {
                return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
            }
        }).then(function (result) {
            cognitiveService = result;
            return cognitiveService.GetUtterancesList(azure.subscriptionRegion, azure.subscriptionKey, app_id);
        }).then(function (utterances) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: utterances });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/utterance/get_utterance/:utterance_id', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var utteranceId = req.params.utterance_id;
        var utterance;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!utteranceId)
                throw 'Missing utterance input parameter';
            return Promise.resolve(true);
        }).then(function (resultUtterance) {
            utterance = resultUtterance;
            return Promise.resolve(true);
        }).then(function (result) {
            return utterance;
        }).then(function (utterance) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: utterance });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/utterance/get_utterances_by_interaction/:audio_id/:section/:part', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var audioId = req.params.audio_id;
        var section = req.params.section;
        var part = req.params.part;
        logger.info('Request parameters: ', { audioId: audioId, section: section, part: part });
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!audioId)
                throw 'Missing audioId input parameter';
            if (!section)
                throw 'Missing section input parameter';
            if (!part)
                throw 'Missing part input parameter';
            return mssql_service_1.MSSql.GetUtterancesByInteraction(vcc, audioId, section, part);
        }).then(function (resultUtterances) {
            logger.info('resultUtterances: ', resultUtterances);
            return Promise.resolve(resultUtterances);
        }).then(function (resultUtterances) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: resultUtterances });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/utterance/create_utterance', function (req, res, next) {
        logger.info('[UtteranceService::create_utterance] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var nlu = null;
        var newUtterance = (req.body || {});
        var createUtteranceResult = null;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!newUtterance.appId)
                throw 'MISSING_NLU_INPUT_PARAMETER';
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, newUtterance.appId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_NLU_NOT_FOUND';
            nlu = result;
            if (nlu.isSystemApp) {
                return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
            }
            else {
                return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
            }
        })
            .then(function (result) {
            cognitiveService = result;
            if (nlu.isSystemApp) {
                return CreateSystemAppIfNotExists(cognitiveService, vcc, nlu);
            }
            else {
                return Promise.resolve(true);
            }
        })
            .then(function (result) {
            return cognitiveService.AddUtterance(azure.subscriptionRegion, azure.subscriptionKey, newUtterance.appId, newUtterance);
        })
            .then(function (resultSaveUtterance) {
            if (!resultSaveUtterance)
                throw 'Se produjo un error al guardar la expresión';
            createUtteranceResult = resultSaveUtterance;
            nlu.needTrain = true;
            nlu.lastModifiedDate = nowUtc;
            nlu.lastModifiedByUserId = userId;
            return nlu_service_1.SaveNLU(vcc, nlu);
        })
            .then(function (resultSaveNLU) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (createUtteranceResult ? 'OK' : 'Utterance save failed'), data: createUtteranceResult });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[UtteranceService::create_utterance] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/utterance/create_utterance_underlined', function (req, res, next) {
        logger.info('[UtteranceService::create_utterance_underlined] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newUtteranceData = (req.body || {});
        newUtteranceData.createdDate = nowUtc;
        newUtteranceData.createdByUserId = userId;
        newUtteranceData.lastModifiedDate = nowUtc;
        newUtteranceData.lastModifiedByUserId = userId;
        logger.info('newUtteranceDTO: ', newUtteranceData);
        promises_shared_1.DummyPromise()
            .then(function (resultSaveUtterance) {
            if (!resultSaveUtterance)
                throw 'Se produjo un error al guardar la expresión';
            logger.info('[UtteranceService::create_utterance_underlined] resultSaveUtterance: ', resultSaveUtterance);
            return mssql_service_1.MSSql.SaveUtterance(vcc, newUtteranceData.appId, newUtteranceData.intentId, newUtteranceData.audioId, newUtteranceData.section, newUtteranceData.part, newUtteranceData.speaker, newUtteranceData.anchorOffset, newUtteranceData.focusOffset, newUtteranceData.currentTime, newUtteranceData.endCurrentTime, newUtteranceData.utteranceId, newUtteranceData.textContent, newUtteranceData.createdDate, newUtteranceData.createdByUserId, newUtteranceData.lastModifiedDate, newUtteranceData.lastModifiedByUserId);
        }).then(function (resultSaveUtteranceUnderlined) {
            logger.info('[UtteranceService::create_utterance_underlined] resultSaveUtteranceUnderlined: ', resultSaveUtteranceUnderlined);
            if (res.headersSent)
                return;
            res.json({ status: true, description: (resultSaveUtteranceUnderlined ? 'OK' : 'Utterance save failed'), data: resultSaveUtteranceUnderlined });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[UtteranceService::create_utterance_underlined] Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/utterance/save_utterance', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newUtterance = (req.body || {});
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return Promise.resolve(true);
        }).then(function (resultSaveUtterance) {
            if (!resultSaveUtterance)
                throw 'Se produjo un error al guardar la expresión';
            return Promise.resolve(true);
        }).then(function (resultSaveUtterance) {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveUtterance, description: (resultSaveUtterance ? 'OK' : 'Utterance save failed'), data: resultSaveUtterance });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/utterance/delete_utterance', function (req, res, next) {
        logger.info('[UtteranceService::delete_utterance] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var utterance = req.body.utterance;
        var utteranceId = utterance.id;
        var intentId = utterance.intentLabel;
        var appId = utterance.appId;
        var nlu = null;
        var deleteUtteranceResult = null;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!appId)
                throw 'MISSING_NLU_INPUT_PARAMETER';
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, appId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_NLU_NOT_FOUND';
            nlu = result;
            if (nlu.isSystemApp) {
                return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
            }
            else {
                return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
            }
        })
            .then(function (result) {
            cognitiveService = result;
            if (!utterance)
                throw 'Missing utterance input parameter';
            return cognitiveService.DeleteAppUtterance(azure.subscriptionRegion, azure.subscriptionKey, appId, intentId, utteranceId);
        }).then(function (resultDeleteUtterance) {
            if (!resultDeleteUtterance)
                throw 'Se produjo un error al eliminar la expresión';
            deleteUtteranceResult = resultDeleteUtterance;
            nlu.needTrain = true;
            nlu.lastModifiedDate = nowUtc;
            nlu.lastModifiedByUserId = userId;
            return nlu_service_1.SaveNLU(vcc, nlu);
        }).then(function (resultDeleteUtterance) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteUtterance, description: (resultDeleteUtterance ? 'OK' : 'Utterance delete failed'), data: resultDeleteUtterance });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[UtteranceService::delete_utterance] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.UtteranceService = UtteranceService;
function CreateSystemAppIfNotExists(cognitiveService, vcc, nlu) {
    var arrAppId = nlu.appId.split('::');
    var systemAppType = arrAppId[1].split('-')[0];
    var isCreated = false;
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        return cognitiveService.GetIntentsList(null, null, nlu.appId);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_GET_INTENTS_FAILED';
        if (result.length === 0) {
            return cognitiveService.AddApp(null, null, nlu);
        }
        else {
            isCreated = true;
            return Promise.resolve(true);
        }
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_CREATE_SYSTEM_MODEL_FAILED';
        if (!isCreated) {
            return cognitiveService.AddIntent(null, null, nlu.appId, 'Agent');
        }
        else {
            return Promise.resolve(true);
        }
    })
        .then(function (result) {
        if (result)
            throw 'SERVER_ERROR_CREATE_SYSTEM_MODEL_INTENT_FAILED';
        return Promise.resolve(true);
    })
        .catch(function (err) {
        var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        logger.error('[UtteranceService::CreateDiarizationAppIfNotExists] Error: ' + errorMsg);
        return Promise.resolve(false);
    });
}
function GetIntent(vcc, name) {
    var cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(function () {
        return mssql_service_1.MSSql.GetIntent(vcc, name);
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        logger.error('Error al ejecutar GetIntent: ' + errMsg);
    });
}
exports.GetIntent = GetIntent;
